#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <time.h>

#include "mex.h"

void lbfgs_(int* n, int* m, double* w, 
	   double* f, double* g, int* diagco, double*  diag, 
	   int* iprint, double* eps, double* xtol, double* wa, 
	   int* iflag);


void Xv(int l, double *x_val, long int *x_jr, long int *x_ic, double *v, double *Xv)
{
	int i, j;

	for(i=0;i<l;i++) 
	{
		Xv[i] = 0;
		for(j=x_ic[i];j<x_ic[i+1];j++)
		  Xv[i] += v[x_jr[j]]*x_val[j];
	}

}
void XTv(int l, int n, double *x_val, long int *x_jr, long int *x_ic, double *v, double *XTv)
{
	int i, j;
	for(i=0;i<n;i++) XTv[i]=0;
	for(i=0;i<l;i++) 
		for(j=x_ic[i];j<x_ic[i+1];j++)
			XTv[x_jr[j]] += x_val[j]*v[i];
}

void test(int ltest, int n, double *w, double *xtest_val, long int *xtest_jr, long int *xtest_ic, double *ytest)
{
	double *pred = (double*) malloc(ltest*sizeof(double));
	Xv(ltest, xtest_val, xtest_jr, xtest_ic, w, pred);
	int correct = 0;
	int i;
	for(i=0; i<ltest; i++)
		if(pred[i] > 0 && ytest[i] > 0)
			correct++;
		else if(pred[i] <= 0 && ytest[i] < 0)
			correct++;
	printf("accuracy %lf\n", (double)correct/ltest);
}
double fg(int l, int n, double *w, double *x_val, long int *x_jr, long int *x_ic, double *y, double *C, double *g)

{
	int i;
	double f=0;
	double *z = (double *) malloc(l*sizeof(double));

	Xv(l, x_val, x_jr, x_ic, w, z);
	for(i=0;i<l;i++) 
		f += C[i]*log(1 + exp(-y[i]*z[i]));

	f = 2*f;
	for(i=0;i<n;i++) 
		f += w[i]*w[i];
	f /= 2.0;

	for(i=0;i<l;i++) 
	{
		z[i] = 1/(1 + exp(-y[i]*z[i]));
		z[i] = C[i]*(z[i]-1)*y[i];
	}
	XTv(l, n, x_val, x_jr, x_ic, z, g);	

	for(i=0;i<n;i++) 
		g[i] = w[i] + g[i];

	free(z);
	return(f);
}

double norm_inf(int n, double *x)
{
	int i;
	double dmax = fabs(x[0]);
	for (i=1; i<n; i++)
		if (fabs(x[i]) >= dmax) 
			dmax = fabs(x[i]);
	return(dmax);
}

double norm_2(int n, double *x)
{
	int i;
	double norm = 0;
	for(i=0;i<n;i++)
		norm += x[i]*x[i];
	return sqrt(norm);
}

void mexFunction(int nlhs, mxArray *plhs[],
		int nrhs, const mxArray *prhs[] )
{
	double *y, *x_val, *C;
	double *ytest, *xtest_val;
	double eps;
	long int *x_jr, *x_ic;
	long int *xtest_jr, *xtest_ic;
	int i, n, l,ltest;

	y = mxGetPr(prhs[0]);
	ytest = mxGetPr(prhs[4]);
	n = mxGetM(prhs[1]);
	l = mxGetN(prhs[1]);
	ltest= mxGetN(prhs[5]);
	x_val = mxGetPr(prhs[1]);
	xtest_val= mxGetPr(prhs[5]);
	x_jr = (long int *) mxGetIr(prhs[1]);
	x_ic = (long int *) mxGetJc(prhs[1]);
	xtest_jr = (long int *) mxGetIr(prhs[5]);
	xtest_ic = (long int *) mxGetJc(prhs[5]);
	C = mxGetPr(prhs[2]);
	eps = mxGetScalar(prhs[3]);

	plhs[0] = mxCreateDoubleMatrix(n, 1, mxREAL) ;
	double *w = mxGetPr(plhs[0]);

	double *g = (double *) malloc(n*sizeof(double));
	double *diag = (double *) malloc(n*sizeof(double));
	double *wa = (double *) malloc((15*n+14)*sizeof(double));

	double f, xtol = 1.0e-16;
	int* iprint = (int*) malloc(2*sizeof(int));
	int diagco = 0;
	int iflag = 0, icall = 0, m = 5;

	iprint[0] = -1; iprint[1] = 0;

	double total_time = 0;
	double clock_start_time;

	for (i=0; i<n; i++) w[i] = 0;

	while(1)
	{
		clock_start_time = clock();
		f = fg(l, n, w, x_val, x_jr, x_ic, y, C, g);
		total_time += clock()-clock_start_time;

		printf("iter %d time %lf f %.12lf g %.12lf ", icall, total_time/CLOCKS_PER_SEC, f, norm_2(n,g)); 
		test(ltest, n, w, xtest_val, xtest_jr, xtest_ic, ytest);
		if (icall > 10000) break;

		icall++;
		clock_start_time = clock();
		lbfgs_(&n, &m, w, &f, g, &diagco, diag, iprint, &eps, &xtol, wa, &iflag);
		total_time += clock()-clock_start_time;
		if (iflag <= 0) break;
	}

	free(g);
	free(diag);
	free(wa);
	free(iprint);
}
